/*
 * ctx_ddl.sql
 * Appendix B, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script contains an example of CTX_DDL
 */

PROMPT Creating the text_search table
PROMPT 

BEGIN
   EXECUTE IMMEDIATE ('DROP TABLE text_search');
EXCEPTION
   WHEN OTHERS
   THEN
      NULL;
END;
/


CREATE TABLE text_search (
   text_search_id NUMBER(10) PRIMARY KEY,
   text VARCHAR2(100));

PROMPT
PROMPT Adding the data to be indexed
PROMPT 

INSERT INTO text_search
   VALUES (1, 'The Bill Of Rights');
INSERT INTO text_search
   VALUES (2, 'The Constitution of the United States');
INSERT INTO text_search
   VALUES (3, 'The Declaration of Independence');
COMMIT;

PROMPT
PROMPT Dropping the preference if it already exists.
PROMPT 

BEGIN
   CTX_DDL.DROP_PREFERENCE('MY_LEXER');
EXCEPTION
   WHEN OTHERS
   THEN
      NULL;
END;
/

PROMPT
PROMPT Creating the preference
PROMPT 

BEGIN
   CTX_DDL.CREATE_PREFERENCE('MY_LEXER', 'BASIC_LEXER');
   CTX_DDL.SET_ATTRIBUTE('MY_LEXER', 'BASE_LETTER', 'YES');
END;
/

PROMPT
PROMPT Creating the index
PROMPT 

CREATE INDEX my_index ON text_search(text)
INDEXTYPE IS CTXSYS.CONTEXT
PARAMETERS ('LEXER my_lexer');

PROMPT 
PROMPT Adding data to mark the index as pending
PROMPT 

INSERT INTO text_search
   VALUES (4, 'The Gettysburg Address');
COMMIT;

PROMPT 
PROMPT Sync pending indexes
PROMPT

SET SERVEROUTPUT ON
CREATE OR REPLACE PROCEDURE index_sync
IS

   CURSOR index_cur
   IS
      SELECT pnd_index_name
      FROM ctx_user_pending;

   v_index_name VARCHAR2(30);

BEGIN

   OPEN index_cur;

   LOOP
      FETCH index_cur INTO v_index_name;
      EXIT WHEN index_cur%NOTFOUND;

      DBMS_OUTPUT.PUT_LINE('Index: '||v_index_name||' is pending');
      -- Sync any indexes returned by the cursor
      CTX_DDL.SYNC_INDEX(v_index_name);

      DBMS_OUTPUT.PUT_LINE('Index: '
                           ||v_index_name
                           ||' has been synchronized');
 
   END LOOP;

   CLOSE index_cur;
 
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.PUT_LINE(SQLERRM);
END index_sync;
/

EXEC index_sync
