/*
 * ctx_report.sql
 * Appendix B, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script contains an example of CTX_REPORT
 */


PROMPT Creating the text_search table
PROMPT 

BEGIN
   EXECUTE IMMEDIATE ('DROP TABLE text_search');
EXCEPTION
   WHEN OTHERS
   THEN
      NULL;
END;
/


CREATE TABLE text_search (
   text_search_id NUMBER(10) PRIMARY KEY,
   text VARCHAR2(100));

PROMPT
PROMPT Adding the data to be indexed
PROMPT 

INSERT INTO text_search
   VALUES (1, 'The Bill Of Rights');
INSERT INTO text_search
   VALUES (2, 'The Constitution of the United States');
INSERT INTO text_search
   VALUES (3, 'The Declaration of Independence');
COMMIT;

PROMPT
PROMPT Dropping the preference if it already exists.
PROMPT 

BEGIN
   CTX_DDL.DROP_PREFERENCE('MY_LEXER');
EXCEPTION
   WHEN OTHERS
   THEN
      NULL;
END;
/

PROMPT
PROMPT Creating the preference
PROMPT 

BEGIN
   CTX_DDL.CREATE_PREFERENCE('MY_LEXER', 'BASIC_LEXER');
   CTX_DDL.SET_ATTRIBUTE('MY_LEXER', 'BASE_LETTER', 'YES');
END;
/

PROMPT
PROMPT Creating the index
PROMPT 

CREATE INDEX my_index ON text_search(text)
INDEXTYPE IS CTXSYS.CONTEXT
PARAMETERS ('LEXER my_lexer');

-- Gather index statistics

CREATE TABLE index_report (
   id number(10), 
   report CLOB);

DECLARE
   v_report CLOB := null;
BEGIN
   CTX_REPORT.INDEX_STATS(
                  index_name => 'MY_INDEX',
                  report => v_report,
                  part_name => NULL,
                  frag_stats => NULL,
                  list_size => 20,
                  report_format => NULL);


   INSERT INTO index_report (id, report)
     VALUES (1, v_report);

   COMMIT;

   DBMS_LOB.FREETEMPORARY(v_report);

END;
/

SET PAGES 9999 HEAD OFF
SPOOL index_stats.txt
SELECT report 
FROM index_report
WHERE id = 1;
SPOOL OFF
