/*
 * HelloWorldThin.java
 * Appendix D, Oracle Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script builds a stand alone Java program.
 */

// Class imports.
import java.sql.*;
import java.io.*;
import java.util.*;

// Oracle class imports.
import oracle.jdbc.driver.*;

// ------------------------------ Begin Class --------------------------------/

// Class definition.
public class HelloWorldThin
{
  // Define a static class String variable.
  private static String user;

  // ------------------------- Begin Constructor -----------------------------/

  /*
  || The constructors of the class are:
  || =========================================================================
  ||  Access     Constructor Type  Constructor
  ||  ---------  ----------------  -------------------------------------------
  ||  public     Default           ThinJDBC()
  */

  // Default constructor.
  public HelloWorldThin()
  {
    // Null constructor.

  } // End of default constructor.

  // -------------------------- End Constructor ------------------------------/

  // --------------------------- Begin Methods -------------------------------/

  /*
  || The static main instantiates a test instance of the class:
  || =========================================================================
  ||  Return Type  Method Name                    Access     Parameter List
  ||  -----------  -----------------------------  ---------  -----------------
  ||  void         printLine()                    private
  ||  void         printLine()                    private    String s
  ||  String       readEntry()                    private
  */

  // -------------------------------------------------------------------------/

  // Print line only method.
  private static void printLine()
  {
    // Call overloaded method with a null String.
    printLine(null);

  } // End of printLine() method.

  // -------------------------------------------------------------------------/

  // Print String with a line method.
  private static void printLine(String s)
  {
    if (s != null)
    {
      // Print the String to console.
      System.out.println(s);

    } // End of if String length is zero.

    // Print a line.
    System.out.print  ("----------------------------------------");
    System.out.println("---------------------------------------");

  } // End of printLine(String s) method.

  // -------------------------------------------------------------------------/

  // Method to read a line from standard input.
  private static String readEntry()
  {
    // Use try-catch to raise exception for a IO exception.
    try
    {
      // Define a int to read a stream.
      int c;

      // Define and intialize a StringBuffer.
      StringBuffer buffer = new StringBuffer();

      // Read first character.
      c = System.in.read();

      // Use a loop to increment and read all characters.
      while (c != '\n' && c != -1)
      {
        // If the character is not an EOL marker append it to the buffer.
        buffer.append((char)c);

        // Read next character.
        c = System.in.read();

      } // End of while character not EOL marker.

      // Return trimmed buffer.
      return buffer.toString().trim();

    } // End of try block.
    catch (IOException e)
    {
      // Return a null if input is a null.
      return null;

    } // End of catch block.

  } // End of readEntry() method.

  // ---------------------------- End Methods --------------------------------/

  // ------------------------- Begin Static Main -----------------------------/

  // Static main to test JDBC connection, with exceptions thrown.
  public static void main(String args[]) throws SQLException, IOException
  {
    // Define and initialize boolean control variables.
    boolean debug = false;

    // Define String variables.
    String userIn;
    String password;
    String host;
    String port;
    String database;
    String debugString = new String("DEBUG");

    // Define int variable.
    int slashIndex;

    // Print line for console output.
    printLine();

    // Validate whether connection is run in debug mode.
    if (args.length > 0)
    {
      // Is the argument a debug instruction.
      if (args[0].toUpperCase().equals(debugString))
      {
        // Enable debug.
        debug = true;

        // Print debug mode enabled.
        printLine("Debug mode is enabled.");

      }
      else
      {
        // Send message on argument.
        for (int i = 0;i < args.length;i++)
        {
          // Print submitted arguments.
          System.out.println("Incorrect argument(s): [" + args[i] + "]");

        } // End of loop to print submitted arguments.

        // Print line break after for-loop because may run more than once.
        printLine();

        // Print valid arguments.
        printLine("Valid case insensitive argument is: DEBUG.");

      } // End of if to validate argument instruction.

    } // End of if to manage any arguments.

    // Load Oracle JDBC driver.
    DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());

    // Prompt the user for connect information.
    System.out.print("Enter User [UID/PASSWD]: ");

    // Get user input.
    userIn = readEntry();

    // Parse user input for token between UID and password.
    slashIndex = userIn.indexOf("/");

    // Check for token in String.
    if (slashIndex != -1)
    {
      // Assign the substring to the left of the token for user.
      user = userIn.substring(0, slashIndex);

      // Assign the substring to the right of the token for password.
      password = userIn.substring(slashIndex + 1);

    }
    else // Token missing from the String.
    {
      // Assign the first entry as user.
      user = userIn;

      // Get user input for password.
      System.out.print("Enter Password: ");

      // Assign the second entry as password.
      password = readEntry();

    } // End of if for token in String.

    // Get user input for password.
    System.out.print("Enter Host Name: ");

    // Assign entry to database name.
    host = readEntry();

    // Get user input for password.
    System.out.print("Enter Port Name: ");

    // Assign entry to database name.
    port = readEntry();

    // Get user input for password.
    System.out.print("Enter Database Name: ");

    // Assign entry to database name.
    database = readEntry();

    // Print message advising progress.
    printLine("Connecting to the database ...");

    // Print JDBC connect string.
    printLine("jdbc:oracle:oci8:@" +
               host + ":" + port + ":" + database + "," +
               user + "," + password);

    // Attempt Oracle8i connection.
    try
    {
      // Define and initialize a JDBC connection.
      Connection conn = DriverManager.getConnection("jdbc:oracle:thin:@" +
                          host + ":" + port + ":" + database, user, password);

      // Print connected.
      printLine("Connected.");

      // Define metadata object.
      DatabaseMetaData dmd = conn.getMetaData();

      // Print database metadata.
      printLine("Driver Version: [" + dmd.getDriverVersion() + "]\n" +
                "Driver Name:    [" + dmd.getDriverName() + "]");

      // Create a statement.
      Statement stmt = conn.createStatement();

      // Execute and return statement.
      ResultSet rset = stmt.executeQuery("SELECT 'Hello World.' FROM dual");

      // Enter a loop for the return set.
      while (rset.next())
      {
        // Print result set.
        printLine(rset.getString(1));

      } // End of loop to read return set.

      // Close the result set.
      rset.close();
      stmt.close();
      conn.close();

      // Print line and acknowledgment.
      printLine("The JDBC Connection worked.");

    } // End of try to establish a connection.
    catch (SQLException e)
    {
      // Print the meaningful error or stacktrace for debug mode.
      if (debug)
      {
        // Use print() because the String returned has a postpended "\n".
        e.printStackTrace();
        printLine();

      }
      else
      {
        // If a mistyped host name, port number or database name.
        if (e.getSQLState() == null)
        {
          // Provide SQL state error message since detail is missing in
          // the classes12.zip archive for SQLException.getErrorCode()
          // returns a 17002.
          System.out.println(
            new SQLException("Oracle Thin Client Net8 Connection Error.",
                      "ORA-" + e.getErrorCode() +
                      ": Incorrect Net8 thin client arguments:\n\n" +
                      "  host name     [" + host + "]\n" +
                      "  port number   [" + port + "]\n" +
                      "  database name [" + database + "]\n",
                      e.getErrorCode()).getSQLState());
        }
        else
        {
          // Trim the postpended "\n".
          printLine(e.getMessage().substring(0,e.getMessage().length() - 1));

        } // End of if mistyped host name, port number or database name.

      } // End of if to print message or StackTrace.

      // Print line and acknowledgment.
      printLine("The JDBC Connection failed.");

    } // End of SQLException catch.

  } // End of static main.

  // -------------------------- End Static Main ------------------------------/

} // End of ThinJDBC class.
