/*
 * create_user.sql
 * Chapter 1, Expert PL/SQL Programming
 * by Ron Hardman and Mike McLaughlin
 *
 * This script creates the plsql user for chapter 1.
 *   You must run this script as SYS or
 *   SYSTEM as SYSDBA.  The script can be rerun.
 *
 * The plsql user is created using the USERS and TEMP tablespace.
 *   If these tablespaces do not exist in your environment, change
 *   the script to use an appropriate tablespaces already created.
 *
 * Modify the conn_string value below, providing your Net Service
 *   Name if not using your default.
 *
 *  DBA'S BEWARE - THE DBA ROLE IS GRANTED TO THE PLSQL USER
 *  IN THIS CHAPTER'S SCRIPT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 */

DEF conn_string = plsql/oracle@book
DEF username = plsql
DEF default_ts = USERS
DEF temp_ts = TEMP

SET FEEDBACK OFF SERVEROUTPUT ON VERIFY OFF TERMOUT OFF

SPOOL create_user.log

DECLARE
   v_count       INTEGER        := 0;
   v_statement   VARCHAR2 (500);
BEGIN

   -- Drop the user if it exists

   SELECT COUNT (1)
     INTO v_count
     FROM dba_users
    WHERE username = UPPER ('&username');

   IF v_count != 0
   THEN
      EXECUTE IMMEDIATE ('DROP USER &username CASCADE');
   END IF;

   v_count := 0;

   -- Create the user

   v_statement :=
         'CREATE USER &username IDENTIFIED BY oracle'
      || ' DEFAULT TABLESPACE &default_ts'
      || ' TEMPORARY TABLESPACE &temp_ts'
      || ' QUOTA UNLIMITED ON &default_ts'
      || ' ACCOUNT UNLOCK';

   EXECUTE IMMEDIATE (v_statement);

   -- Grant permissions
   EXECUTE IMMEDIATE ('GRANT connect, resource TO &username');
   EXECUTE IMMEDIATE ('GRANT DBA TO &username');

   DBMS_OUTPUT.put_line ('	');
   DBMS_OUTPUT.put_line ('User &username created successfully');
   DBMS_OUTPUT.put_line ('	');


EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line (SQLERRM);
      DBMS_OUTPUT.put_line ('	');
END;
/

SET FEEDBACK ON TERMOUT ON

PROMPT
PROMPT Connecting to user &username to create a maintenance package
PROMPT

CONN &conn_string


CREATE OR REPLACE PACKAGE clean_schema IS

   PROCEDURE tables;
 
END;
/


CREATE OR REPLACE PACKAGE BODY clean_schema AS

PROCEDURE tables
IS
   v_string VARCHAR2(50);
   v_tab_error EXCEPTION;
   PRAGMA EXCEPTION_INIT(v_tab_error, -942); 
BEGIN
   BEGIN
      v_string := 'DROP TABLE airport_list';
   EXECUTE IMMEDIATE (v_string);
   EXCEPTION
   WHEN v_tab_error
      THEN
      NULL;
   END;

   BEGIN
      v_string := 'DROP TABLE assessment';
   EXECUTE IMMEDIATE (v_string);
   EXCEPTION
   WHEN v_tab_error
      THEN
      NULL;
   END;

END tables;

END;
/


SPOOL OFF
