/*
 * edit_distance.sql
 * Chapter 1, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script demonstrates a 10gR2 new feature - UTL_MATCH
 */

SPOOL edit_distance.log

CREATE OR REPLACE FUNCTION edit_distance
   (i_source_string IN VARCHAR2,
    i_target_string IN VARCHAR2)
   RETURN NUMBER
   DETERMINISTIC
AS

   /*******************************************************
    * The edit_distance function was originally authored
    * by Barbara Boehmer, and graciously provided to Expert
    * PL/SQL to use as a comparison with UTL_MATCH.
    *******************************************************/
    
   v_length_of_source NUMBER := NVL (LENGTH (i_source_string), 0);
   v_length_of_target NUMBER := NVL (LENGTH (i_target_string), 0);

   TYPE mytabtype IS TABLE OF NUMBER INDEX BY BINARY_INTEGER;
   
   column_to_left mytabtype;
   current_column mytabtype;
   
   v_cost NUMBER := 0;
   
BEGIN
  IF v_length_of_source = 0 THEN
    RETURN v_length_of_target;
  ELSIF v_length_of_target = 0 THEN
    RETURN v_length_of_source;
  ELSE
    FOR j IN 0 .. v_length_of_target LOOP
      column_to_left(j) := j;
    END LOOP;
    FOR i IN 1.. v_length_of_source LOOP
      current_column(0) := i;
      FOR j IN 1 .. v_length_of_target LOOP
        IF SUBSTR (i_source_string, i, 1) =
          SUBSTR (i_target_string, j, 1)
          THEN v_cost := 0;
        ELSE v_cost := 1;
        END IF;
        current_column(j) := LEAST (current_column(j - 1) + 1,
                                    column_to_left(j) + 1,
                                    column_to_left(j - 1) + v_cost);
      END LOOP;
      FOR j IN 0 .. v_length_of_target LOOP
        column_to_left(j) := current_column(j);
      END LOOP;
    END LOOP;
  END IF;
  RETURN current_column(v_length_of_target);
END edit_distance;
/ 

PROMPT 
PROMPT *** 
PROMPT *** Tests the EDIT_DISTANCE function
PROMPT ***
PROMPT
SELECT EDIT_DISTANCE('expresso', 'espresso') AS DISTANCE
FROM dual;

PROMPT 
PROMPT *** 
PROMPT *** Tests the UTL_MATCH.EDIT_DISTANCE function
PROMPT ***
PROMPT
SELECT UTL_MATCH.EDIT_DISTANCE('expresso', 'espresso') AS DISTANCE
FROM dual;

PROMPT 
PROMPT *** 
PROMPT *** Tests the UTL_MATCH.EDIT_DISTANCE_SIMILARITY function
PROMPT ***
PROMPT
SELECT UTL_MATCH.EDIT_DISTANCE_SIMILARITY('expresso', 'espresso') AS DISTANCE
FROM dual;

spool off