/*
 * wrap.sql
 * Chapter 1, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script demonstrates a 10gR2 new feature - Wrapping Dynamic PL/SQL
 */

spool wrap.log

exec CLEAN_SCHEMA.tables

CREATE TABLE airport_list (
                           airport_id NUMBER(10) PRIMARY KEY,
                           airport_code VARCHAR2(10 CHAR) NOT NULL,
                           region VARCHAR2(30 CHAR) NOT NULL,
                           country VARCHAR2(30 CHAR) NOT NULL);

SET FEEDBACK OFF

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (1, 'AKL', 'Auckland', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (2, 'BHE', 'Blenheim', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (3, 'CHC', 'Christchurch', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (4, 'CHT', 'Chatham Islands', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (5, 'DUD', 'Dunedin', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (6, 'GIS', 'Gisborne', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (7, 'HLZ', 'Hamilton', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (8, 'HKK', 'Hokitika', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (9, 'IVC', 'Invercargill', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (10, 'KAT', 'Kaitaia', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (11, 'KKE', 'Kerikeri', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (12, 'NPE', 'Napier/Hastings', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (13, 'NPL', 'New Plymouth', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (14, 'NSN', 'Nelson', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (15, 'PMR', 'Palmerston North', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (16, 'ROT', 'Rotorua', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (17, 'SZS', 'Stewart Island', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (18, 'TIU', 'Timaru', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (19, 'TRG', 'Tauranga', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (20, 'TUO', 'Taupo', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (21, 'WKA', 'Wanaka', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (22, 'WAG', 'Wanganui', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (23, 'WLG', 'Wellington', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (24, 'WHK', 'Whakatane', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (25, 'WRE', 'Whangarei', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (26, 'WSZ', 'Westport', 'NEW ZEALAND');

INSERT INTO airport_list (airport_id, airport_code, region, country)
VALUES (27, 'ZQN', 'Queenstown', 'NEW ZEALAND');

COMMIT;

SET FEEDBACK ON

-- Uses DBMS_DDL.WRAP to obfuscate PL/SQL source
PROMPT
PROMPT * * * 
PROMPT * * * Use the WRAP function to wrap the source
PROMPT * * *
PROMPT

DECLARE
v_procedure VARCHAR2(32767);
BEGIN
  v_procedure := 'CREATE OR REPLACE PROCEDURE wrap_test '
  ||'IS '
  ||'   v_airport_codes AIRPORT_LIST.AIRPORT_CODE%TYPE; '
  ||' '
  ||'   CURSOR airport_cur IS '
  ||'      SELECT airport_code'
  ||'        FROM airport_list'
  ||'        ORDER BY airport_code;'
  ||' '
  ||'BEGIN '
  ||'   FOR y IN airport_cur LOOP '
  ||'      DBMS_OUTPUT.PUT_LINE(''Airport Code: ''||y.airport_code);'
  ||'   END LOOP;'
  || 'END;';
  
  EXECUTE IMMEDIATE DBMS_DDL.WRAP(v_procedure);

END;
/

SET PAGES 9999 LONG 64000

PROMPT
PROMPT * * *
PROMPT * * * Wrapped procedure...
PROMPT * * *
PROMPT

SELECT text
FROM user_source
WHERE name = 'WRAP_TEST';

PROMPT
PROMPT * * *
PROMPT * * * Procedure results...
PROMPT * * *
PROMPT 

SET SERVEROUTPUT ON

EXEC wrap_test




-- Uses DBMS_DDL.CREATE_WRAPPED to obfuscate PL/SQL source
PROMPT
PROMPT
PROMPT * * * 
PROMPT * * * Use the CREATE_WRAPPED procedure to wrap the source
PROMPT * * *
PROMPT

DECLARE
v_procedure VARCHAR2(32767);
BEGIN
  v_procedure := 'CREATE OR REPLACE PROCEDURE create_wrapped_test '
  ||'IS '
  ||'   v_airport_codes AIRPORT_LIST.AIRPORT_CODE%TYPE; '
  ||' '
  ||'   CURSOR airport_cur IS '
  ||'      SELECT airport_code'
  ||'        FROM airport_list'
  ||'        ORDER BY airport_code;'
  ||' '
  ||'BEGIN '
  ||'   FOR y IN airport_cur LOOP '
  ||'      DBMS_OUTPUT.PUT_LINE(''Airport Code: ''||y.airport_code);'
  ||'   END LOOP;'
  || 'END;';
  
  SYS.DBMS_DDL.CREATE_WRAPPED(v_procedure);

END;
/

SET PAGES 9999 LONG 64000

PROMPT
PROMPT * * *
PROMPT * * * Wrapped procedure...
PROMPT * * *
PROMPT

SELECT text
FROM user_source
WHERE name = 'CREATE_WRAPPED_TEST';

PROMPT
PROMPT * * *
PROMPT * * * Procedure results...
PROMPT * * *
PROMPT 

SET SERVEROUTPUT ON

EXEC create_wrapped_test

SPOOL OFF