/*
 * basic_error3.sql
 * Chapter 3, Oracle10g PL/SQL Programming
 * by Ron Hardman, Michael McLaughlin and Scott Urman
 *
 * This script raises and manages a standard anonymous block PL/SQL
 * program error.
 */

-- Unremark for debugging script.
-- SET ECHO ON
SET SERVEROUTPUT ON SIZE 1000000

-- Drop any objects to make script re-runnable.
BEGIN

  FOR i IN (SELECT   table_name
            FROM     user_tables
            WHERE    table_name = 'DUMMY' ) LOOP
            
    -- Use NDS to drop the dependent object type.
    EXECUTE IMMEDIATE 'DROP TABLE dummy';
    
  END LOOP;
  
END;
/

-- Create a dummy table.
CREATE TABLE dummy
( dummy_value       VARCHAR2(1));

-- Build an anonymous block that will trigger an error.
DECLARE

  -- Define local variable.
  my_string         VARCHAR2(1) := ' '; 
  my_number         NUMBER;

BEGIN

  -- Select a white space into a local variable.
  SELECT ' ' INTO my_string FROM dummy;

  -- Attempt to ssign a single white space to a number.
  my_number := TO_NUMBER(my_string);

EXCEPTION

  WHEN no_data_found THEN
    dbms_output.put_line('SELECT-INTO'||CHR(10)||SQLERRM);

END;
/
