/*
 * create_objects.sql
 * Chapter 3, Oracle10g PL/SQL Programming
 * by Ron Hardman, Michael McLaughlin and Scott Urman
 *
 * This script verifies and defines the PLSQL user. It should be run
 * as the SYSTEM user or a user that has the DBA privilege role and
 * EXECUTE with grant option on DBMS_PIPE.
 */

-- Unremark for debugging script.
-- SET ECHO ON
SET SERVEROUTPUT ON SIZE 1000000

-- Create a nested table of numbers
CREATE OR REPLACE TYPE number_type AS TABLE OF NUMBER;
/

-- Create a nested table of strings.
CREATE OR REPLACE TYPE varchar2_type AS TABLE OF NUMBER;
/

-- Drop any objects to make script re-runnable.
BEGIN

  FOR i IN (SELECT   table_name
            FROM     user_tables
            WHERE    table_name = 'APP_ERROR' ) LOOP
            
    -- Use NDS to drop the dependent object type.
    EXECUTE IMMEDIATE 'DROP TABLE app_error';
    
  END LOOP;
  
END;
/

-- Create an alert_error table to track raised trigger errors
CREATE TABLE app_error
( error_id                    NUMBER       NOT NULL
, module_name                 VARCHAR2(30) NOT NULL
, table_name                  VARCHAR2(30)
, class_name                  VARCHAR2(30)
, message_string              VARCHAR2(80)
, sqlerrm_string              VARCHAR2(80)
, last_update_date            DATE         NOT NULL
, last_updated_by             NUMBER       NOT NULL
, creation_date               DATE         NOT NULL
, created_by                  NUMBER       NOT NULL
, CONSTRAINT pk_error_id      PRIMARY KEY (error_id)
);


