/*
 * create_invoker_data_store.sql
 * Chapter 4, Oracle Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script creates a table and package to be used in
 * a code repository example of invoker's rights.
 */

-- Unremark for debugging script.
-- SET ECHO ON
SET SERVEROUTPUT ON SIZE 1000000

-- Dropy synonym if it exists.
BEGIN

  FOR i IN (SELECT   NULL
            FROM     user_synonyms
            WHERE    synonym_name = 'DATA_MANAGEMENT') LOOP

    EXECUTE IMMEDIATE 'DROP SYNONYM data_management';

  END LOOP;

END;
/

-- Create synonym to the PLSQL.DATA_MANAGEMENT package.
CREATE SYNONYM data_management FOR plsql.data_management;

-- Dropy synonym if it exists.
BEGIN

  FOR i IN (SELECT   NULL
            FROM     user_tables
            WHERE    table_name = 'MYAPP_DATA_STORE') LOOP

    EXECUTE IMMEDIATE 'DROP TABLE myapp_data_store';

  END LOOP;

END;
/

-- Create local table.
CREATE TABLE myapp_data_store
( data_id       NUMBER
, data_text     VARCHAR2(20 CHAR));

-- Create a local view that mirrors the code repository.
CREATE OR REPLACE VIEW data_store AS
SELECT   *
FROM     myapp_data_store;

-- Call the invoker's right stored package.
EXECUTE data_management.insert_data(1,'One');

-- Use SQL*Plus to format the output.
COL data_text FORMAT A20

-- Query the local MYAPP_DATA_STORE table.
SELECT   data_id
,        data_text
FROM     myapp_data_store;
