/*
 * create_striped_view1.sql
 * Chapter 4, Oracle Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script builds a view that enables you to use
 * DBMS_APPLICATION_INFO and USERENV() in building a
 * publish-and-subscribe implementation. It should be
 * run in the PLSQL user schema.
 */

-- Unremark for debugging script.
-- SET ECHO ON
SET SERVEROUTPUT ON SIZE 1000000

-- Drop shared table.
DROP TABLE shared_all;

-- Create shared table.
CREATE TABLE shared_all
( shared_id     NUMBER
, shared_text   VARCHAR2(20 CHAR)
, striping_id   VARCHAR2(10 CHAR));

-- Select the default value from V$SESSION.CLIENT_INFO.
CREATE OR REPLACE VIEW shared AS
  SELECT   shared_id
  ,        shared_text
  FROM     shared_all
  WHERE    NVL(striping_id,0) =
             NVL(TO_NUMBER(SUBSTR(USERENV('CLIENT_INFO'),1,10)),0);

-- Grant all on the shared view to an external schema.
GRANT ALL ON shared TO MYAPP;

-- Insert a non-striped row.
INSERT
INTO     shared_all
VALUES
( 1,'One','');

-- Insert a striped row.
INSERT
INTO     shared_all
VALUES
( 2,'Two','1');

-- Insert a striped row.
INSERT
INTO     shared_all
VALUES
( 3,'Three','2');