/*
 * subscribing.sql
 * Chapter 4, Oracle Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script tests a view that enables you to use
 * DBMS_APPLICATION_INFO and USERENV() in building a
 * publish-and-subscribe implementation. It should be
 * run in the MYAPP user schema.

 */

-- Unremark for debugging script.
-- SET ECHO ON
SET NULL "<Null>"
SET SERVEROUTPUT ON SIZE 1000000

-- Use a PL/SQL block to check before dropping a synonym.
BEGIN

  FOR i IN (SELECT   NULL
            FROM     user_synonyms
            WHERE    synonym_name = 'SHARED') LOOP

    EXECUTE IMMEDIATE 'DROP SYNONYM shared';

  END LOOP;

END;
/

-- Unset the CLIENT_INFO value.
EXECUTE DBMS_APPLICATION_INFO.SET_CLIENT_INFO(NULL);

-- Create synonym to the striped view.
CREATE SYNONYM shared FOR plsql.shared;

-- Select the default value from V$SESSION.CLIENT_INFO.
SELECT   USERENV('CLIENT_INFO')
FROM     dual;

-- Format column returns.
COL shared_id   FORMAT 990
COL shared_text FORMAT A20
COL userenv_col FORMAT A10

-- Select the non-striped values from the view.
SELECT   shared_id
,        shared_text
,        USERENV('CLIENT_INFO') userenv_col
FROM     shared;

-- Execute the DBMS_APPLICATION_INFO package.
EXECUTE DBMS_APPLICATION_INFO.SET_CLIENT_INFO('1');

-- Select the overridden value from V$SESSION.CLIENT_INFO.
SELECT   shared_id
,        shared_text
,        USERENV('CLIENT_INFO') userenv_col
FROM     shared;
