/*
 * create_basic_object2.sql
 * Chapter 6, Oracle Expert PL/SQL
 * by Ron Hardman and Michael McLaughlin
 *
 * This script builds a sample object type with an
 * ORDER member function.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL create_basic_object2.log

SET ECHO ON
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF

-- Define an object type.
CREATE OR REPLACE TYPE build_test
AUTHID CURRENT_USER IS OBJECT
( my_number NUMBER
, my_name   VARCHAR2(20 CHAR)
, CONSTRUCTOR FUNCTION build_test
RETURN SELF AS RESULT 
, CONSTRUCTOR FUNCTION build_test
( my_number NUMBER
, my_name   VARCHAR2 )
RETURN SELF AS RESULT 
, MEMBER PROCEDURE print_instance_variable
, ORDER MEMBER FUNCTION equals
( my_class BUILD_TEST ) RETURN NUMBER )
INSTANTIABLE NOT FINAL;
/

SHOW ERRORS

-- Build the object type body.
CREATE OR REPLACE TYPE BODY build_test AS

  -- Define a null argument constructor.
  CONSTRUCTOR FUNCTION build_test
  RETURN SELF AS RESULT IS

    -- Define an instance variable.
    my_instance_number NUMBER := 0;
    my_instance_name   VARCHAR2(20 CHAR) := '';

  BEGIN

    -- Assign the local variable to the instance variable.
    SELF.my_number := my_instance_number;
    SELF.my_name := my_instance_name;

    -- Return instance of the object type.
    RETURN;

  END;

  -- Define a single argument constructor.
  CONSTRUCTOR FUNCTION build_test
  ( my_number NUMBER 
  , my_name   VARCHAR2 )
  RETURN SELF AS RESULT IS

  BEGIN

    -- Assign the parameters to instance variables.
    SELF.my_number := my_number;
    SELF.my_name := my_name;

    -- Return instance of the object type.
    RETURN;

  END;

  -- An object member procedure.
  MEMBER PROCEDURE print_instance_variable IS

  BEGIN

    -- Print the instance variable.
    DBMS_OUTPUT.PUT_LINE('Instance Number ['||SELF.my_number||']');
    DBMS_OUTPUT.PUT_LINE('Instance Name   ['||SELF.my_name||']');

  END;

  -- A sequencing member procedure.
  ORDER MEMBER FUNCTION equals
  ( my_class BUILD_TEST )
  RETURN NUMBER IS

    -- Define a default return values.
    false_value NUMBER := 0;
    true_value  NUMBER := 1;

  BEGIN

    -- Determine value to return.
    IF SELF.my_number = my_class.my_number AND
       SELF.my_name = my_class.my_name     THEN

      -- Return True they are equal.
      RETURN true_value;

    ELSE 

      -- Return False they are unequal.
      RETURN false_value;

    END IF;
      
  END;

END;
/

SHOW ERRORS

SPOOL OFF
