/*
 * create_sample_object1.sql
 * Chapter 6, Oracle Expert PL/SQL
 * by Ron Hardman and Michael McLaughlin
 *
 * This script builds a sample object type.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL create_sample_object1.log

SET ECHO ON
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF

-- Define an object type.
CREATE OR REPLACE TYPE name_object
AUTHID CURRENT_USER IS OBJECT
( first_name VARCHAR2(20 CHAR)
, last_name  VARCHAR2(20 CHAR)
, CONSTRUCTOR FUNCTION name_object
RETURN SELF AS RESULT 
, CONSTRUCTOR FUNCTION name_object
( first_name VARCHAR2
, last_name  VARCHAR2 )
RETURN SELF AS RESULT 
, MEMBER PROCEDURE print_instance_variable
, ORDER MEMBER FUNCTION equals
( my_class NAME_OBJECT ) RETURN NUMBER )
INSTANTIABLE NOT FINAL;
/

SHOW ERRORS

-- Build the object type body.
CREATE OR REPLACE TYPE BODY name_object AS

  -- Define a null argument constructor.
  CONSTRUCTOR FUNCTION name_object
  RETURN SELF AS RESULT IS

    -- Define an instance variable.
    first_name VARCHAR2(20 CHAR) := NULL;
    last_name  VARCHAR2(20 CHAR) := NULL;

  BEGIN

    -- Assign the local variable to the instance variable.
    SELF.first_name := first_name;
    SELF.last_name := last_name;

    -- Return instance of the object type.
    RETURN;

  END;

  -- Define a single argument constructor.
  CONSTRUCTOR FUNCTION name_object
  ( first_name VARCHAR2
  , last_name  VARCHAR2 )
  RETURN SELF AS RESULT IS

  BEGIN

    -- Assign the parameters to instance variables.
    SELF.first_name := first_name;
    SELF.last_name := last_name;

    -- Return instance of the object type.
    RETURN;

  END;

  -- An object member procedure.
  MEMBER PROCEDURE print_instance_variable IS

  BEGIN

    -- Print the instance variable.
    DBMS_OUTPUT.PUT_LINE(
      'Name ['||SELF.first_name||', '||SELF.last_name||']');

  END;

  -- A sequencing member procedure.
  ORDER MEMBER FUNCTION equals
  ( my_class NAME_OBJECT )
  RETURN NUMBER IS

    -- Define a default return values.
    false_value NUMBER := 0;
    true_value  NUMBER := 1;

  BEGIN

    -- Determine value to return.
    IF SELF.first_name = my_class.first_name AND
       SELF.last_name = my_class.last_name   THEN

      -- Return True they are equal.
      RETURN true_value;

    ELSE 

      -- Return False they are unequal.
      RETURN false_value;

    END IF;
      
  END;

END;
/

SHOW ERRORS

SPOOL OFF
