/*
 * create_sample_object2.sql
 * Chapter 6, Oracle Expert PL/SQL
 * by Ron Hardman and Michael McLaughlin
 *
 * This script builds a sample object type and tests it.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL create_sample_object2.log

SET ECHO ON
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF

-- Define an object type.
CREATE OR REPLACE TYPE contact_object
AUTHID CURRENT_USER IS OBJECT
( contact_name   NAME_OBJECT
, street_address VARCHAR2(20 CHAR)
, city           VARCHAR2(14 CHAR)
, state          VARCHAR2(2 CHAR)
, postal_code    VARCHAR2(5 CHAR)
, CONSTRUCTOR FUNCTION contact_object
RETURN SELF AS RESULT 
, CONSTRUCTOR FUNCTION contact_object
( contact_name   NAME_OBJECT
, street_address VARCHAR2
, city           VARCHAR2
, state          VARCHAR2
, postal_code    VARCHAR2 )
RETURN SELF AS RESULT
, CONSTRUCTOR FUNCTION contact_object
( first_name     VARCHAR2
, last_name      VARCHAR2
, street_address VARCHAR2
, city           VARCHAR2
, state          VARCHAR2
, postal_code    VARCHAR2 )
RETURN SELF AS RESULT
, MEMBER PROCEDURE print_instance_variable
, ORDER MEMBER FUNCTION equals
( my_class CONTACT_OBJECT ) RETURN NUMBER )
INSTANTIABLE NOT FINAL;
/

SHOW ERRORS

-- Build the object type body.
CREATE OR REPLACE TYPE BODY contact_object AS

  -- Define a null argument constructor.
  CONSTRUCTOR FUNCTION contact_object
  RETURN SELF AS RESULT IS

    -- Define an instance variable.
    first_name     VARCHAR2(20 CHAR) := NULL;
    last_name      VARCHAR2(20 CHAR) := NULL;
    street_address VARCHAR2(20 CHAR) := NULL;
    city           VARCHAR2(14 CHAR) := NULL;
    state          VARCHAR2(2 CHAR)  := NULL;
    postal_code    VARCHAR2(5 CHAR)  := NULL;

  BEGIN

    -- Assign the local variable to the instance variable.
    SELF.contact_name := name_object(first_name,last_name);
    SELF.street_address := street_address;
    SELF.city := city;
    SELF.state := state;
    SELF.postal_code := postal_code;

    -- Return instance of the object type.
    RETURN;

  END;

  -- Define a compound object type argument constructor.
  CONSTRUCTOR FUNCTION contact_object
  ( contact_name   NAME_OBJECT
  , street_address VARCHAR2
  , city           VARCHAR2
  , state          VARCHAR2
  , postal_code    VARCHAR2 )
  RETURN SELF AS RESULT IS

  BEGIN

    -- Assign the parameters to instance variables.
    SELF.contact_name := contact_name;
    SELF.street_address := street_address;
    SELF.city := city;
    SELF.state := state;
    SELF.postal_code := postal_code;

    -- Return instance of the object type.
    RETURN;

  END;

  -- Define a compound object type argument constructor.
  CONSTRUCTOR FUNCTION contact_object
  ( first_name     VARCHAR2
  , last_name      VARCHAR2
  , street_address VARCHAR2
  , city           VARCHAR2
  , state          VARCHAR2
  , postal_code    VARCHAR2 )
  RETURN SELF AS RESULT IS

  BEGIN

    -- Assign the parameters to instance variables.
    SELF.contact_name := name_object(first_name,last_name);
    SELF.street_address := street_address;
    SELF.city := city;
    SELF.state := state;
    SELF.postal_code := postal_code;

    -- Return instance of the object type.
    RETURN;

  END;

  -- An object member procedure.
  MEMBER PROCEDURE print_instance_variable IS

  BEGIN

    -- Print the instance variables from the embedded object type.
    SELF.contact_name.print_instance_variable;

    -- Print a formatting line.
    DBMS_OUTPUT.PUT_LINE('-------------------------------');
    DBMS_OUTPUT.PUT_LINE('First Name    : '||SELF.contact_name.first_name);
    DBMS_OUTPUT.PUT_LINE('Last Name     : '||SELF.contact_name.last_name);
    DBMS_OUTPUT.PUT_LINE('Street Address: '||SELF.street_address);
    DBMS_OUTPUT.PUT_LINE('City          : '||SELF.city);
    DBMS_OUTPUT.PUT_LINE('State         : '||SELF.state);
    DBMS_OUTPUT.PUT_LINE('Postal Code   : '||SELF.postal_code);

  END;

  -- A sequencing member procedure.
  ORDER MEMBER FUNCTION equals
  ( my_class CONTACT_OBJECT )
  RETURN NUMBER IS

    -- Define a default return values.
    false_value NUMBER := 0;
    true_value  NUMBER := 1;

  BEGIN

    -- Determine value to return.
    IF SELF.contact_name.equals(my_class.contact_name) = true_value AND
       SELF.street_address = my_class.street_address AND
       SELF.city = my_class.city AND
       SELF.state = my_class.state AND
       SELF.postal_code = my_class.postal_code THEN

      -- Return True they are equal.
      RETURN true_value;

    ELSE 

      -- Return False they are unequal.
      RETURN false_value;

    END IF;
      
  END;

END;
/

SHOW ERRORS

SPOOL OFF
