/*
 * create_tables.sql
 * Chapter 6, Oracle Expert PL/SQL
 * by Ron Hardman and Michael McLaughlin
 *
 * This script builds the data tables for Chapter 6 and seeds
 * them with data.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL create_tables.log

SET ECHO ON
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF

DROP TABLE application_user CASCADE CONSTRAINTS;

CREATE TABLE application_user
( user_id              NUMBER
, user_name            VARCHAR2(20 CHAR)
, user_group_id        NUMBER
, user_type            NUMBER
, last_name            VARCHAR2(20 CHAR)
, first_name           VARCHAR2(20 CHAR)
, middle_initial       VARCHAR2(1 CHAR)
, CONSTRAINT           user_id_u1
  PRIMARY KEY         (user_id)
);

INSERT
INTO application_user
VALUES
( 1
,'SYSADMIN'
, 1
, 1
,''
,''
,'');

DROP SEQUENCE application_user_s1;

CREATE SEQUENCE application_user_s1
INCREMENT BY    1
START WITH      1001
ORDER;

DROP TABLE common_lookup CASCADE CONSTRAINTS;

CREATE TABLE common_lookup
( lookup_id            NUMBER
, lookup_context       VARCHAR2(30 CHAR)
, lookup_type          VARCHAR2(30 CHAR)
, lookup_meaning       VARCHAR2(30 CHAR)
, CONSTRAINT           common_lookup_u1
  PRIMARY KEY         (lookup_id)
);

CREATE        INDEX comlookup_n1
ON     common_lookup ( lookup_context );

CREATE UNIQUE INDEX comlookup_u2
ON     common_lookup ( lookup_context, lookup_type );

DROP SEQUENCE common_lookup_s1;

CREATE SEQUENCE common_lookup_s1
INCREMENT BY    1
START WITH      1001
ORDER;

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'APP_USER'
,'ADMINISTRATOR'
,'System Administrator');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'CONTACT'
,'PERSONAL_CONTACT'
,'Personal Contact');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'CONTACT'
,'WORK_CONTACT'
,'Work Contact');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'ADDRESS'
,'HOME_ADDRESS'
,'Home Address');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'ADDRESS'
,'WORK_ADDRESS'
,'Home Address');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'TELEPHONE'
,'HOME_PHONE'
,'Home Phone');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'TELEPHONE'
,'HOME_FAX'
,'Home Fax');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'TELEPHONE'
,'WORK_PHONE'
,'Home Phone');

INSERT
INTO     common_lookup
VALUES
( common_lookup_s1.nextval
,'TELEPHONE'
,'WORK_FAX'
,'Work Fax');

DROP TABLE contact CASCADE CONSTRAINTS;

CREATE TABLE contact
( contact_id           NUMBER
, contact_group_id     NUMBER
, contact_type         NUMBER
, last_name            VARCHAR2(20 CHAR)
, first_name           VARCHAR2(20 CHAR)
, middle_initial       VARCHAR2(1 CHAR)
, CONSTRAINT           contact_u1
  PRIMARY KEY         (contact_id)
);

CREATE        INDEX contact_n1
ON     contact ( contact_group_id );

DROP SEQUENCE contact_s1;

CREATE SEQUENCE contact_s1
INCREMENT BY    1
START WITH      1001
ORDER;

DROP SEQUENCE contact_s2;

CREATE SEQUENCE contact_s2
INCREMENT BY    1
START WITH      1001
ORDER;

DROP TABLE address CASCADE CONSTRAINTS;

CREATE TABLE address
( address_id           NUMBER
, contact_id           NUMBER
, address_type         NUMBER
, city                 VARCHAR2(30 CHAR)
, state_province       VARCHAR2(30 CHAR)
, postal_code          VARCHAR2(20 CHAR)
, CONSTRAINT           address_u1
  PRIMARY KEY         (address_id)
);

CREATE        INDEX address_n1
ON     address ( contact_id );

DROP SEQUENCE address_s1;

CREATE SEQUENCE address_s1
INCREMENT BY    1
START WITH      1001
ORDER;

DROP TABLE street_address CASCADE CONSTRAINTS;

CREATE TABLE street_address
( street_address_id    NUMBER
, address_line_number  NUMBER
, address_id           NUMBER
, street_address       VARCHAR2(30 CHAR)
, CONSTRAINT           street_address_u1
  PRIMARY KEY         (street_address_id)
);

CREATE        INDEX street_address_n1
ON     street_address ( address_id );

DROP SEQUENCE street_address_s1;

CREATE SEQUENCE street_address_s1
INCREMENT BY    1
START WITH      1001
ORDER;

DROP TABLE telephone CASCADE CONSTRAINTS;

CREATE TABLE telephone
( telephone_id         NUMBER
, contact_id           NUMBER
, address_id           NUMBER
, telephone_type       NUMBER
, country_code         VARCHAR2(3 CHAR)
, area_code            VARCHAR2(6 CHAR)
, telephone_number     VARCHAR2(10 CHAR)
, CONSTRAINT           telephone_u1
  PRIMARY KEY         (telephone_id)
);

CREATE        INDEX telephone_n1
ON     telephone ( contact_id, address_id );

CREATE        INDEX telephones_n2
ON     telephone ( address_id );

DROP SEQUENCE telephone_s1;

CREATE SEQUENCE telephone_s1
INCREMENT BY    1
START WITH      1001
ORDER;

INSERT
INTO     contact
VALUES
( contact_s1.nextval
, contact_s2.nextval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'CONTACT'
  AND      lookup_type = 'PERSONAL_CONTACT')
,'Smith'
,'Robert'
,'B');

INSERT
INTO     address
VALUES
( address_s1.nextval
, contact_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'ADDRESS'
  AND      lookup_type = 'HOME_ADDRESS')
,'San Carlos'
,'CA'
,'94070');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 1
, address_s1.currval
,'2006 Sylvan');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 2
, address_s1.currval
,'Apartment 12A');

INSERT
INTO     telephone
VALUES
( telephone_s1.nextval
, contact_s1.currval
, address_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'TELEPHONE'
  AND      lookup_type = 'HOME_PHONE')
,'001'
,'650'
,'111.1111');

INSERT
INTO     telephone
VALUES
( telephone_s1.nextval
, contact_s1.currval
, address_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'TELEPHONE'
  AND      lookup_type = 'HOME_FAX')
,'001'
,'650'
,'111.2222');

INSERT
INTO     contact
VALUES
( contact_s1.nextval
, contact_s2.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'CONTACT'
  AND      lookup_type = 'PERSONAL_CONTACT')
,'Smith'
,'Jamie'
,'H');

INSERT
INTO     address
VALUES
( address_s1.nextval
, contact_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'ADDRESS'
  AND      lookup_type = 'HOME_ADDRESS')
,'San Carlos'
,'CA'
,'94070');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 1
, address_s1.currval
,'2006 Sylvan');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 2
, address_s1.currval
,'Apartment 12A');

INSERT
INTO     telephone
VALUES
( telephone_s1.nextval
, contact_s1.currval
, address_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'TELEPHONE'
  AND      lookup_type = 'HOME_PHONE')
,'001'
,'650'
,'111.1111');

INSERT
INTO     telephone
VALUES
( telephone_s1.nextval
, contact_s1.currval
, address_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'TELEPHONE'
  AND      lookup_type = 'HOME_FAX')
,'001'
,'650'
,'111.2222');

INSERT
INTO     contact
VALUES
( contact_s1.nextval
, contact_s2.nextval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'CONTACT'
  AND      lookup_type = 'WORK_CONTACT')
,'Curddle'
,'George'
,'W');

INSERT
INTO     address
VALUES
( address_s1.nextval
, contact_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'ADDRESS'
  AND      lookup_type = 'WORK_ADDRESS')
,'Redwood Shores'
,'CA'
,'94070');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 1
, address_s1.currval
,'300 Oracle Parkway');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 2
, address_s1.currval
,'Suite 1045');

INSERT
INTO     contact
VALUES
( contact_s1.nextval
, contact_s2.nextval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'CONTACT'
  AND      lookup_type = 'WORK_CONTACT')
,'Book'
,'Ida'
,'W');

INSERT
INTO     address
VALUES
( address_s1.nextval
, contact_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'ADDRESS'
  AND      lookup_type = 'WORK_ADDRESS')
,'Redwood Shores'
,'CA'
,'94070');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 1
, address_s1.currval
,'500 Oracle Parkway');

INSERT
INTO     street_address
VALUES
( street_address_s1.nextval
, 2
, address_s1.currval
,'Suite 5032');

INSERT
INTO     telephone
VALUES
( telephone_s1.nextval
, contact_s1.currval
, address_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'TELEPHONE'
  AND      lookup_type = 'WORK_PHONE')
,'001'
,'650'
,'222.2222');

INSERT
INTO     telephone
VALUES
( telephone_s1.nextval
, contact_s1.currval
, address_s1.currval
,(SELECT   lookup_id
  FROM     common_lookup
  WHERE    lookup_context = 'TELEPHONE'
  AND      lookup_type = 'WORK_FAX')
,'001'
,'650'
,'222.1111');

COMMIT;

COL address   FORMAT A30

SELECT   c1.first_name||DECODE(c2.first_name,'','',' and ')
||       c2.first_name||' '||c1.last_name||CHR(10)
||       sa1.street_address||CHR(10)||sa2.street_address||CHR(10)
||       a.city||', '||a.state_province||' '||a.postal_code address
FROM     contact c1
,        contact c2
,        address a
,        street_address sa1
,        street_address sa2
WHERE    c1.contact_group_id = c2.contact_group_id(+)
AND      c1.contact_id != c2.contact_id(+)
AND      c1.contact_id = a.contact_id
AND      a.address_id = sa1.address_id
AND      sa1.address_line_number = 1
AND      a.address_id = sa2.address_id
AND      sa2.address_line_number = 2;

SPOOL OFF
