
/*
 * compiler_warnings.sql
 * Chapter 8, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script demonstrates both PLSQL_WARNINGS and 
 *  DBMS_WARNING
 */

spool compiler_warnings.log

/*  Do not run script as-is.  ALTER SYSTEM commands must be run as sysdba
    USE AS REFERENCE ONLY.

PROMPT ***
PROMPT *** PLSQL_COMPILER setting
PROMPT ***

SHOW PARAMETER PLSQL_WARNINGS

PROMPT ***
PROMPT *** Enable warnings for PERFORMANCE
PROMPT ***


ALTER SYSTEM SET PLSQL_WARNINGS='ENABLE:PERFORMANCE', 'ENABLE:SEVERE';

PROMPT ***
PROMPT *** PLSQL_COMPILER setting
PROMPT ***

SHOW PARAMETER PLSQL_WARNINGS

PROMPT ***
PROMPT *** Create a procedure that generates a warning
PROMPT ***

CREATE OR REPLACE PROCEDURE seed_data
IS
   v_age PERSON.AGE%TYPE;
   v_birth_date PERSON.DOB%TYPE;
   v_month_shift NUMBER(10);
   v_person_id PERSON.PERSON_ID%TYPE := 1;
   v_gender PERSON.GENDER%TYPE := 'M';
   v_first_name PERSON.FIRST_NAME%TYPE := 'Hodge';
   v_last_name PERSON.LAST_NAME%TYPE := 'Podge';
   v_activity_id ACTIVITY.ACTIVITY_ID%TYPE;
   v_hair_color PERSON.HAIR_COLOR%TYPE := 'BROWN';
   v_eye_color PERSON.EYE_COLOR%TYPE := 'BLUE';
BEGIN
  FOR y IN 1..200000
    LOOP
    FOR x IN 1..2
      LOOP
      FOR z IN 1..12
        LOOP
        -- Includes ages 31  42
        v_age := 30 + z;

        -- Set the value to deduct from today's date to determine the bday
        v_month_shift := v_age * - 12;

        -- Determine the bday based on today's date and the age
        v_birth_date := ADD_MONTHS(systimestamp, v_age * - 12); 

        -- Person_ID must be unique
        v_person_id := v_person_id + 1;

        -- Pick random activities
        v_activity_id := ROUND(DBMS_RANDOM.VALUE(1, 12), 0); 

        INSERT INTO person
        VALUES (v_person_id, v_first_name || v_person_id, v_last_name 
                || v_person_id, 
                v_age, v_gender, v_birth_date, v_hair_color, v_eye_color,
                'bio text for a person '|| v_age ||' years old', 
                v_activity_id, v_person_id - 1);
      END LOOP;

      -- change gender and name so 1/2 of entries are female
      v_gender := 'F';
      v_first_name := 'Hither';
      v_last_name := 'Yonder';
      v_hair_color := 'BLONDE';
      v_eye_color := 'BROWN';

      COMMIT;
    END LOOP;
  END LOOP;
EXCEPTION
   WHEN OTHERS
   THEN 
      DBMS_OUTPUT.PUT_LINE(DBMS_UTILITY.FORMAT_ERROR_STACK);
END;
/


SHOW ERRORS


PROMPT ***
PROMPT *** Check the object's status
PROMPT ***

COL object_name FORMAT A30
COL status FORMAT A10
SELECT object_name, status
  FROM dba_objects
  WHERE object_name = 'SEED_DATA'
  AND owner = 'PLSQL';


PROMPT ***
PROMPT *** Generate an error instead...
PROMPT ***

ALTER PROCEDURE plsql.seed_data COMPILE PLSQL_WARNINGS='ERROR:07202';


PROMPT ***
PROMPT *** Check the object's status
PROMPT ***

COL object_name FORMAT A30
COL status FORMAT A10
SELECT object_name, status
  FROM dba_objects
  WHERE object_name = 'SEED_DATA'
  AND owner = 'PLSQL';

PROMPT ***
PROMPT *** Disable all warnings
PROMPT ***

ALTER SYSTEM SET PLSQL_WARNINGS='DISABLE:ALL';


PROMPT ***
PROMPT *** USE DBMS_WARNING INSTEAD
PROMPT ***

PROMPT 
PROMPT ***
PROMPT *** Get the warning level
PROMPT ***

SELECT DBMS_WARNING.GET_WARNING_SETTING_STRING() WARNING_LEVEL
FROM dual;

PROMPT ***
PROMPT *** Enable all warnings
PROMPT ***

CALL DBMS_WARNING.SET_WARNING_SETTING_STRING('ENABLE:ALL', 'SESSION');

PROMPT ***
PROMPT *** Recompile the procedure
PROMPT ***

alter procedure plsql.seed_data compile;
show errors

PROMPT ***
PROMPT *** Create a procedure that doesn't generate a warning
PROMPT ***

CREATE OR REPLACE PROCEDURE plsql.seed_data
IS
   v_age PERSON.AGE%TYPE;
   v_birth_date PERSON.DOB%TYPE;
   v_month_shift NUMBER(10);
   v_person_id PERSON.PERSON_ID%TYPE := 1;
   v_gender PERSON.GENDER%TYPE := 'M';
   v_first_name PERSON.FIRST_NAME%TYPE := 'Hodge';
   v_first_name2 PERSON.FIRST_NAME%TYPE;
   v_last_name PERSON.LAST_NAME%TYPE := 'Podge';
   v_last_name2 PERSON.LAST_NAME%TYPE;
   v_activity_id ACTIVITY.ACTIVITY_ID%TYPE;
BEGIN
  FOR y IN 1..200000
    LOOP
    FOR x IN 1..2
      LOOP
      FOR z IN 1..12
        LOOP

        -- Includes ages 31  42
        v_age := 30 + z;

        -- Set the value to deduct from today's date to determine the bday
        v_month_shift := v_age * - 12;

        -- Determine the bday based on today's date and the age
        v_birth_date := ADD_MONTHS(systimestamp, v_age * - 12); 

        -- Person_ID must be unique
        v_person_id := v_person_id + 1;

        -- Pick random activities
        v_activity_id := ROUND(DBMS_RANDOM.VALUE(1, 12), 0); 
        
        -- Set first_name, last_name, and bio values
	v_first_name2 := v_first_name||v_person_id;
	v_last_name2 := v_last_name||v_person_id;


        INSERT INTO person
        VALUES (v_person_id, 
                v_first_name2, 
                v_last_name2, 
                v_age, v_gender, v_birth_date, 
                'bio text', 
                v_activity_id, v_person_id - 1);
      END LOOP;

    -- change gender and name so 1/2 of entries are female
    v_gender := 'F';
    v_first_name := 'Hither';
    v_last_name := 'Yonder';
    END LOOP;
    commit;
  END LOOP;
EXCEPTION
   WHEN OTHERS
   THEN 
      DBMS_OUTPUT.PUT_LINE(DBMS_UTILITY.FORMAT_ERROR_STACK);
END;
/

SHOW ERRORS

*/

spool off
