/*
 * profiler.sql
 * Chapter 8, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script demonstrates the DBMS_PROFILER package
 */

spool profiler.log

exec clean_schema.seq
exec clean_schema.tables

SET PAUSE ON

CREATE TABLE users (
   ID NUMBER(10) PRIMARY KEY,
   USERNAME VARCHAR2(20),
   PASSWORD VARCHAR2(500));
   
CREATE SEQUENCE user_seq
START WITH 1
INCREMENT BY 1;

BEGIN
   FOR y IN 1..10000
   LOOP
      INSERT INTO users (id, username, password)
        VALUES (user_seq.NEXTVAL,
                DBMS_RANDOM.STRING('A', 20),
                DBMS_RANDOM.STRING('A', 10));
   END LOOP;
COMMIT;
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.PUT_LINE(DBMS_UTILITY.FORMAT_ERROR_STACK);
END;
/
   
CREATE OR REPLACE FUNCTION encrypt_password (
   i_password IN VARCHAR2)
   RETURN VARCHAR2
IS
   v_ciphertext RAW(500);
   v_deciphered VARCHAR2(500 CHAR);
   v_plaintext RAW(200) := UTL_I18N.STRING_TO_RAW(
                                      i_password,
                                      'AL32UTF8');
   v_type BINARY_INTEGER := DBMS_CRYPTO.AES_CBC_PKCS5;
   v_key RAW(128) := UTL_I18N.STRING_TO_RAW(
            DBMS_CRYPTO.RANDOMBYTES(16),  --generates 128 bit key..128/8
            'AL32UTF8');
BEGIN
   v_ciphertext := DBMS_CRYPTO.ENCRYPT(v_plaintext,
                                       v_type,
                                       v_key);
   RETURN v_ciphertext;
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.PUT_LINE(DBMS_UTILITY.FORMAT_ERROR_STACK);
END;
/

PROMPT 
PROMPT ***
PROMPT *** If you have not yet run $ORACLE_HOME/rdbms/admin/proftab.sql
PROMPT *** do so in another session before continuing.
PROMPT ***
PROMPT *** Press <ENTER> to continue
PROMPT ***
PROMPT
PAUSE

EXEC DBMS_PROFILER.START_PROFILER('encrypt')

DECLARE
   v_new_password USERS.PASSWORD%TYPE;

   CURSOR password_cur
   IS
      SELECT *
      FROM users;
BEGIN
   FOR y IN password_cur
   LOOP
      v_new_password := ENCRYPT_PASSWORD(y.password);
      
      UPDATE users
      SET password = v_new_password
      WHERE id = y.id;
   END LOOP;
END;
/

EXEC DBMS_PROFILER.STOP_PROFILER

PROMPT
PROMPT ***
PROMPT *** Run $ORACLE_HOME/plsql/demo/profsum.sql to see a report
PROMPT ***
PROMPT

SET PAUSE OFF

spool off