/*
 * seed_data.sql
 * Chapter 8, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script adds tables and data for chapter 8 examples
 */

SPOOL seed_data.log

exec clean_schema.tables

CREATE TABLE person (
                     PERSON_ID NUMBER(10),
                     FIRST_NAME VARCHAR2(20 CHAR),
                     LAST_NAME VARCHAR2(20 CHAR),
                     AGE NUMBER(10), 
                     GENDER VARCHAR2(1 CHAR),
                     DOB TIMESTAMP,
                     HAIR_COLOR VARCHAR2(20),
                     EYE_COLOR VARCHAR2(20),
                     BIO VARCHAR2(50 CHAR),
                     ACTIVITY_ID NUMBER(10),
                     REFERRED_BY NUMBER(10));

-- Insert the first user
INSERT INTO person
VALUES (0, 'Original', 'Member', 30, 'F', ADD_MONTHS(systimestamp,  - 480),
        'BROWN', 'BLUE',
        'Fuzzy wuzzy was a bear',
        13, NULL);

CREATE TABLE activity (
                       ACTIVITY_ID NUMBER (10),
                       ACTIVITY_NAME VARCHAR2(50 CHAR));


INSERT INTO activity (activity_id, activity_name)
VALUES (1, 'Running');

INSERT INTO activity (activity_id, activity_name)
VALUES (2, 'Biking');

INSERT INTO activity (activity_id, activity_name)
VALUES (3, 'Boxing');

INSERT INTO activity (activity_id, activity_name)
VALUES (4, 'Sitting');

INSERT INTO activity (activity_id, activity_name)
VALUES (5, 'Doing Handstands');

INSERT INTO activity (activity_id, activity_name)
VALUES (6, 'Cartwheels');

INSERT INTO activity (activity_id, activity_name)
VALUES (7, 'Fishing');

INSERT INTO activity (activity_id, activity_name)
VALUES (8, 'Writing on chalkboards');

INSERT INTO activity (activity_id, activity_name)
VALUES (9, 'Hanging out with the guys');

INSERT INTO activity (activity_id, activity_name)
VALUES (10, 'Sewing');

INSERT INTO activity (activity_id, activity_name)
VALUES (11, 'Changing my car''s oil');

INSERT INTO activity (activity_id, activity_name)
VALUES (12, 'TV...lots of TV');

INSERT INTO activity (activity_id, activity_name)
VALUES (13, 'Anything and everything');
COMMIT;

PROMPT
PROMPT ***
PROMPT *** This will take a significant amount of time as we are inserting
PROMPT ***  4.8 Million records
PROMPT ***
PROMPT 

DECLARE
   v_age PERSON.AGE%TYPE;
   v_birth_date PERSON.DOB%TYPE;
   v_month_shift NUMBER(10);
   v_person_id PERSON.PERSON_ID%TYPE := 1;
   v_gender PERSON.GENDER%TYPE := 'M';
   v_first_name PERSON.FIRST_NAME%TYPE := 'Hodge';
   v_last_name PERSON.LAST_NAME%TYPE := 'Podge';
   v_activity_id ACTIVITY.ACTIVITY_ID%TYPE;
   v_hair_color PERSON.HAIR_COLOR%TYPE := 'BROWN';
   v_eye_color PERSON.EYE_COLOR%TYPE := 'BLUE';

BEGIN
  
  FOR y IN 1..200000
    LOOP
    FOR x IN 1..2
      LOOP
      FOR z IN 1..12
        LOOP
        
        -- Includes ages 31 - 42
        v_age := 30 + z;
        
        -- Set the value to deduct from today's date to determine the bday
        v_month_shift := v_age * - 12;
        
        -- Determine the bday based on today's date and the age
        v_birth_date := ADD_MONTHS(systimestamp, v_age * - 12); 
        
        -- Person_ID must be unique
        v_person_id := v_person_id + 1;
        
        -- Pick random activities
        v_activity_id := ROUND(DBMS_RANDOM.VALUE(1, 12), 0); 
        
        INSERT INTO person
        VALUES (v_person_id, v_first_name || v_person_id, v_last_name || v_person_id, 
                v_age, v_gender, v_birth_date, v_hair_color, v_eye_color,
                'bio text for a person '|| v_age ||' years old', 
                v_activity_id, v_person_id - 1);
      END LOOP;
    
      -- change gender and name so 1/2 of entries are female
      v_gender := 'F';
      v_first_name := 'Hither';
      v_last_name := 'Yonder';
      v_hair_color := 'BLONDE';
      v_eye_color := 'BROWN';
    
      COMMIT;
    END LOOP;
  END LOOP;
EXCEPTION
   WHEN OTHERS
   THEN 
      DBMS_OUTPUT.PUT_LINE(DBMS_UTILITY.FORMAT_ERROR_STACK);
END;
/ 


UPDATE person
SET dob = systimestamp
WHERE person_id IN (479966, 479967);

commit;

spool off