/*
 * tune_sql.sql
 * Chapter 8, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script tests different indexes and the performance
 *  when there is low cardinality.
 */

spool tune_sql.log


PROMPT ***
PROMPT *** Slow - full table scan
PROMPT ***

SET TIMING ON SERVEROUTPUT ON
DECLARE
   v_count NUMBER(10);
BEGIN
   SELECT COUNT(rowid)
   INTO v_count
   FROM person
   WHERE gender = 'F'
   AND activity_id = '11'
   AND hair_color = 'BLONDE'
   AND age = 34
   AND eye_color = 'BROWN';
   DBMS_OUTPUT.PUT_LINE(v_count);
END;
/

PROMPT ***
PROMPT *** Create a bitmap index -- this will take some time
PROMPT ***

DROP INDEX gender_idx;
CREATE BITMAP INDEX person_idx 
ON person(gender, 
          activity_id, 
          hair_color, 
          age, 
          eye_color);

PROMPT ***
PROMPT *** Fastest - using bitmap index
PROMPT ***


SET TIMING ON SERVEROUTPUT ON
DECLARE
   v_count NUMBER(10);
BEGIN
   SELECT COUNT(rowid)
   INTO v_count
   FROM person
   WHERE gender = 'F'
   AND activity_id = '11'
   AND hair_color = 'BLONDE'
   AND age = 34
   AND eye_color = 'BROWN';
   DBMS_OUTPUT.PUT_LINE(v_count);
END;
/

spool off