/*
 * create_text_indexes.sql
 * Chapter 9, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script creates all Oracle Text database objects for the examples.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

spool create_text_indexes.log

SET ECHO ON
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF


-- Create a new stoplist named EXPERT_STOPLIST.  Most of the 
--  words are the same as the DEFAULT_STOPLIST, but there are
--  a few differences.  For example, the word SHALL is used
--  frequently in the documents, and provides no benefit to a
--  the relevance of any document, so it has been added.

begin
  ctx_ddl.create_stoplist('EXPERT_STOPLIST','BASIC_STOPLIST');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','Mr');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','Mrs');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','Ms');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','Mz');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','a');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','about');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','after');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','all');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','also');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','an');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','and');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','any');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','are');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','as');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','at');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','be');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','because');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','been');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','but');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','by');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','can');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','co');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','could');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','for');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','from');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','had');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','has');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','have');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','he');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','her');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','his');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','if');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','in');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','into');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','is');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','it');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','its');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','last');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','more');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','most');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','no');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','not');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','of');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','on');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','one');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','only');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','or');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','other');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','out');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','over');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','s');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','says');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','shall');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','she');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','so');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','some');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','such');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','than');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','that');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','the');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','their');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','there');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','they');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','this');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','to');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','up');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','was');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','we');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','were');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','when');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','which');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','who');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','will');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','with');
  ctx_ddl.add_stopword('EXPERT_STOPLIST','would');
end;
/

-- Create new wordlist preference

BEGIN
   CTX_DDL.CREATE_PREFERENCE('EXPERT_WORDLIST', 'BASIC_WORDLIST');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_WORDLIST', 'FUZZY_MATCH', 'ENGLISH');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_WORDLIST', 'FUZZY_SCORE', '0');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_WORDLIST', 'PREFIX_INDEX', 'TRUE');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_WORDLIST', 'PREFIX_MIN_LENGTH', '2');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_WORDLIST', 'PREFIX_MAX_LENGTH', '5');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_WORDLIST', 'SUBSTRING_INDEX', 'TRUE');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_WORDLIST', 'STEMMER', 'ENGLISH');
END;
/


-- Create a concatenated datastore.  Using this datastore
--  a search on short_description will actually search title as well.

BEGIN
   CTX_DDL.CREATE_PREFERENCE('EXPERT_CONCAT_DATASTORE', 'MULTI_COLUMN_DATASTORE');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_CONCAT_DATASTORE', 
                         'columns', 
                         'TITLE, SHORT_DESCRIPTION');
END;
/

-- Create a storage preference.  Note that it prompts for a
--  tablespace name.  Using the datastore preference I am 
--  making it possible to define a separate tablespace
--  for the Oracle Text support tables (as opposed to just
--  dropping them into the default tablespace).

BEGIN
   ctx_ddl.create_preference('EXPERT_STORAGE', 'BASIC_STORAGE'); 
   ctx_ddl.set_attribute('EXPERT_STORAGE', 
                      'I_TABLE_CLAUSE', 
                      'tablespace &&text_tablespace storage (
                             initial 4K 
                             next 8M 
                             maxextents unlimited 
                             pctincrease 0)'); 
   ctx_ddl.set_attribute('EXPERT_STORAGE', 
                      'K_TABLE_CLAUSE', 
                      'tablespace &&text_tablespace storage (
                             initial 4K 
                             next 8M 
                             maxextents unlimited 
                             pctincrease 0)'); 
   ctx_ddl.set_attribute('EXPERT_STORAGE', 
                      'R_TABLE_CLAUSE', 
                      'lob (data) store as (cache)
                       tablespace &&text_tablespace storage (
                             initial 4K 
                             next 8M 
                             maxextents unlimited 
                             pctincrease 0)'); 
   ctx_ddl.set_attribute('EXPERT_STORAGE', 
                      'I_INDEX_CLAUSE', 
                      'COMPRESS 2
                       tablespace &&text_tablespace storage (
                             initial 4K 
                             next 8M 
                             maxextents unlimited 
                             pctincrease 0)'); 
END;
/

-- Create a lexer using BASIC_LEXER

BEGIN
   CTX_DDL.CREATE_PREFERENCE('EXPERT_LEXER', 'BASIC_LEXER');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_LEXER', 'INDEX_THEMES', 'YES');
   CTX_DDL.SET_ATTRIBUTE('EXPERT_LEXER', 'THEME_LANGUAGE', 'ENGLISH');
END;
/

-- Create a WORLD_LEXER
BEGIN
   CTX_DDL.CREATE_PREFERENCE('EXPERT_WORLD_LEXER', 'WORLD_LEXER');
END;
/

PROMPT -- PLEASE BE PATIENT - THIS MAY TAKE SOME TIME

-- Create a CONTEXT index on the document column

CREATE INDEX EXPERT_IDX ON document_repository(document)
INDEXTYPE IS CTXSYS.CONTEXT
PARAMETERS (
  'LEXER EXPERT_LEXER
   WORDLIST EXPERT_WORDLIST
   STOPLIST EXPERT_STOPLIST
   DATASTORE CTXSYS.DEFAULT_DATASTORE'
     )
/

-- Create a CTXCAT index on the title column

CREATE INDEX expert_title_idx ON document_repository(title)
INDEXTYPE IS CTXSYS.CTXCAT
PARAMETERS (
   'LEXER EXPERT_LEXER
    WORDLIST EXPERT_WORDLIST
    STOPLIST EXPERT_STOPLIST'
      )
/

-- Create an index using the concatenated datastore.  This 
--  actually concatenates short_description and title columns.

CREATE INDEX concat_idx
ON document_repository(short_description)
INDEXTYPE IS CTXSYS.CONTEXT
PARAMETERS('
   DATASTORE EXPERT_CONCAT_DATASTORE
   WORDLIST EXPERT_WORDLIST
   LEXER EXPERT_LEXER
   STOPLIST EXPERT_STOPLIST')
/

-- Create an index on an XML document using the CONTEXT index.  I
--  am using auto section group creation (see the parameters).  Note
--  that the xml doc is not stored using XMLTYPE datatype or indexed
--  using the CTXXPATH index.  It is not necessary for this purpose.

CREATE INDEX xml_idx
ON document_repository(xml_doc)
INDEXTYPE IS CTXSYS.CONTEXT
PARAMETERS ('
   SECTION GROUP CTXSYS.AUTO_SECTION_GROUP
   DATASTORE CTXSYS.DEFAULT_DATASTORE
   WORDLIST EXPERT_WORDLIST
   LEXER EXPERT_LEXER
   STOPLIST EXPERT_STOPLIST')
/

spool off