/*
 * gist.sql
 * Chapter 9, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script generates the gist of a document
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

spool gist.log

SET ECHO ON
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF

SET SERVEROUTPUT ON SIZE 1000000

DECLARE
   v_gist CLOB;
   v_offset PLS_INTEGER := 1;
   v_amount PLS_INTEGER := 32767;
   v_buffer VARCHAR2(32767);
   v_rowid  ROWID;

BEGIN

   EXECUTE IMMEDIATE('truncate table expert_gist');
   CTX_DOC.SET_KEY_TYPE('ROWID'); 

   SELECT rowid
   INTO v_rowid
   FROM document_repository
   WHERE document_id = 1;

   CTX_DOC.GIST(
      index_name => 'EXPERT_IDX',
      textkey => v_rowid,
      restab => 'EXPERT_GIST',
      query_id => 1,
      glevel => 'P',
      pov => 'GENERIC');

   SELECT gist
   INTO v_gist
   FROM expert_gist;

   BEGIN
      LOOP
         DBMS_LOB.READ(
            lob_loc => v_gist, 
            amount => v_amount, 
            offset => v_offset, 
            buffer => v_buffer);

         DBMS_OUTPUT.PUT_LINE(SUBSTR(v_buffer, 1, 255));

         v_offset := v_offset + v_amount;
         v_amount := 32767;
      END LOOP;
   EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      NULL;
   END;
END;
/

spool off