/*
 * plsql_queries.sql
 * Chapter 9, Expert PL/SQL Programming
 * by Ron Hardman, Mike McLaughlin
 *
 * This script demonstrates Oracle Text searches using PL/SQL
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

spool plsql_queries.log

SET ECHO ON
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF

SET SERVEROUTPUT ON

/* To get an exact match, do nothing special.  Pass the string
     without modification. */

BEGIN

   DBMS_OUTPUT.PUT_LINE('EXACT MATCH SEARCH USING FOR LOOP');
   DBMS_OUTPUT.PUT_LINE('=================================');

   FOR c IN (SELECT SCORE(1) "RANK", title
          FROM document_repository
          WHERE CONTAINS(document, 'defence', 1) > 0
          ORDER BY 1 DESC)
   LOOP
      DBMS_OUTPUT.PUT_LINE('-');
      DBMS_OUTPUT.PUT_LINE('Score: '||c.rank);
      DBMS_OUTPUT.PUT_LINE('Title: '||c.title);
   END LOOP;

END;
/


/* Wildcard uses a % before and/or after the search string */

BEGIN

   DBMS_OUTPUT.PUT_LINE('WILDCARD SEARCH USING FOR LOOP');
   DBMS_OUTPUT.PUT_LINE('=================================');

   FOR c IN (SELECT SCORE(1) "RANK", title
          FROM document_repository
          WHERE CONTAINS(document, 'defen%', 1) > 0
          ORDER BY 1 DESC)
   LOOP
      DBMS_OUTPUT.PUT_LINE('-');
      DBMS_OUTPUT.PUT_LINE('Score: '||c.rank);
      DBMS_OUTPUT.PUT_LINE('Title: '||c.title);
   END LOOP;

END;
/

/* Stemming uses a $ preceding the search term(s) */

BEGIN

   DBMS_OUTPUT.PUT_LINE('STEMMING SEARCH USING FOR LOOP');
   DBMS_OUTPUT.PUT_LINE('==============================');

   FOR c IN (SELECT SCORE(1) "RANK", title
          FROM document_repository
          WHERE CONTAINS(document, '$defense', 1) > 0
          ORDER BY 1 DESC)
   LOOP
      DBMS_OUTPUT.PUT_LINE('-');
      DBMS_OUTPUT.PUT_LINE('Score: '||c.rank);
      DBMS_OUTPUT.PUT_LINE('Title: '||c.title);
   END LOOP;

END;
/


/* Use REPLACE to change space to comma delimited */

DECLARE
   v_search_string VARCHAR2(100 CHAR);
BEGIN

   -- original search string is made of two words
   v_search_string := 'defence congress';

   -- replace the white space with a comma
   v_search_string := REPLACE(v_search_string, ' ', ', ');

   DBMS_OUTPUT.PUT_LINE('MULTI-TERM SEARCH USING FOR LOOP');
   DBMS_OUTPUT.PUT_LINE('=================================');

   FOR c IN (SELECT SCORE(1) "RANK", title
          FROM document_repository
          WHERE CONTAINS(document, v_search_string, 1) > 0
          ORDER BY 1 DESC)
   LOOP
      DBMS_OUTPUT.PUT_LINE('-');
      DBMS_OUTPUT.PUT_LINE('Score: '||c.rank);
      DBMS_OUTPUT.PUT_LINE('Title: '||c.title);
   END LOOP;

END;
/


/* Search an XML document using section searching and the WITHIN operator */
BEGIN

   DBMS_OUTPUT.PUT_LINE('ABOUT SEARCH USING FOR LOOP');
   DBMS_OUTPUT.PUT_LINE('===========================');

   FOR c IN (SELECT SCORE(1) "RANK", title
          FROM document_repository
          WHERE CONTAINS(document, 'ABOUT(law)', 1) > 0
          ORDER BY 1 DESC)
   LOOP
      DBMS_OUTPUT.PUT_LINE('-');
      DBMS_OUTPUT.PUT_LINE('Score: '||c.rank);
      DBMS_OUTPUT.PUT_LINE('Title: '||c.title);
   END LOOP;

END;
/


/* Search an XML document using section searching and the WITHIN operator */
BEGIN

   DBMS_OUTPUT.PUT_LINE('SECTION SEARCH USING FOR LOOP');
   DBMS_OUTPUT.PUT_LINE('=================================');

   FOR c IN (SELECT SCORE(1) "RANK", title
          FROM document_repository
          WHERE CONTAINS(xml_doc, 'President within TEXT', 1) > 0
          ORDER BY 1 DESC)
   LOOP
      DBMS_OUTPUT.PUT_LINE('-');
      DBMS_OUTPUT.PUT_LINE('Score: '||c.rank);
      DBMS_OUTPUT.PUT_LINE('Title: '||c.title);
   END LOOP;

END;
/

spool off