 /*
 * cursors.sql
 * Chapter 10, Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script demonstrates how regular expressions can be used
 *  with cursors in PL/SQL, and provide case-insensitive searches
 *  without the use of the UPPER/LOWER functions.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL cursors.log

SET ECHO OFF
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF
SET SERVEROUTPUT ON

@@seed_data.sql
@@create_objects.sql

DECLARE
v_common_name PLANTS.COMMON_NAME%TYPE;
v_first_name BUYERS.FIRST_NAME%TYPE;
v_last_name BUYERS.LAST_NAME%TYPE;
v_order_date ORDERS.ORDER_DATE%TYPE;

CURSOR order_cur IS
SELECT p.common_name, b.first_name, b.last_name, o.order_date
FROM orders o, buyers b, plants p
WHERE o.buyer_id = b.buyer_id
AND o.plant_id = p.plant_id
AND REGEXP_LIKE(p.common_name, 'clemat(is|us)', 'i');

BEGIN
  OPEN order_cur;
  
  LOOP
    FETCH order_cur INTO v_common_name, 
    v_first_name, 
    v_last_name, 
    v_order_date;
    EXIT WHEN order_cur%NOTFOUND;
    
    DBMS_OUTPUT.PUT_LINE('============================');
    DBMS_OUTPUT.PUT_LINE(v_first_name ||' '|| v_last_name
                         ||' ordered '|| v_common_name
                         ||' on '|| v_order_date);
  END LOOP;
  
  DBMS_OUTPUT.PUT_LINE('============================');
  
  CLOSE order_cur;

EXCEPTION
  WHEN OTHERS
    THEN
    DBMS_OUTPUT.PUT_LINE(DBMS_UTILITY.FORMAT_ERROR_STACK);
    CLOSE order_cur;
END;
/ 

SPOOL OFF