/*
 * display_phone_numbers.sql
 * Chapter 10, Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script creates a function that returns a formatted phone number
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL display_phone_numbers.log

SET ECHO OFF
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF
SET SERVEROUTPUT ON

@@seed_data.sql
@@create_objects.sql
 
CREATE OR REPLACE FUNCTION display_phone_numbers (
   i_buyer_id IN buyers.buyer_id%TYPE)
   RETURN VARCHAR2
IS
   v_phone_number VARCHAR2(30 CHAR);
   v_error VARCHAR2(30 CHAR);
BEGIN
 
   --Retrieve the phone number from the buyers table
   SELECT phone
   INTO v_phone_number
   FROM buyers
   WHERE buyer_id = i_buyer_id;
   
   --Format the number
   v_phone_number :=
      REGEXP_REPLACE(v_phone_number, 
          '^([[:digit:]]{3})([[:digit:]]{3})([[:digit:]]{4})$', 
          '(\1) \2-\3');
           
   RETURN v_phone_number;
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.PUT_LINE(DBMS_UTILITY.FORMAT_ERROR_STACK);
END;
/


PROMPT
PROMPT *** Show all buyer names and formatted phone numbers
PROMPT

DECLARE

   CURSOR buyer_cur IS
   SELECT buyer_id, first_name, last_name, 
          phone AS "Phone"
   FROM buyers;
   
BEGIN

   FOR buyer_rec IN buyer_cur
   LOOP
      DBMS_OUTPUT.PUT_LINE(buyer_rec.last_name
                           ||', '||buyer_rec.first_name
                           ||CHR(9)
                           ||display_phone_numbers(buyer_rec.buyer_id));
      DBMS_OUTPUT.PUT(CHR(10));
   END LOOP;
   
END;
/

spool off