/*
 * triggers.sql
 * Chapter 10, Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script creates a trigger that verifies e-mail format on insert
 */

@@seed_data.sql
@@create_objects.sql

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

SPOOL triggers.log

SET ECHO OFF
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF
SET SERVEROUTPUT ON

CREATE OR REPLACE TRIGGER buyer_trig
BEFORE INSERT OR UPDATE ON buyers
FOR EACH ROW
DECLARE
v_email VARCHAR2(50 CHAR);
v_verify BOOLEAN;
BEGIN
  IF :NEW.email IS NOT NULL
    THEN
    v_verify := REGEXP_LIKE(
                   :NEW.email,
                   '^[[:alnum:]](([_\.]?'
                   ||'[[:alnum:]]+)*)@([[:alnum:]]+)'
                   ||'(([\.-]?[[:alnum:]]+)*)\.'
                   ||'([[:alpha:]]{2,})$');
    
    IF NOT v_verify
      THEN
      DBMS_OUTPUT.PUT_LINE('email address: '||:NEW.email);
      RAISE_APPLICATION_ERROR( - 20001, HANDLE_ERRORS.GET_CODE(1));
    END IF;
  END IF;
END;
/

PROMPT
PROMPT *** INSERT OF ron.hardman@plsqlbook.com SUCCEEDS ***
PROMPT

INSERT INTO buyers
VALUES (100, 'Ron', 'Hardman', 'ronhardman@plsqlbook.com', '2918804554');

PROMPT
PROMPT *** INSERT OF m@mclaughlin@plsqlbook.com FAILS ***
PROMPT

INSERT INTO buyers
VALUES (101, 'Michael', 'McLaughlin', 'm@mclaughlin@plsqlbook.com', '3392081098');

ROLLBACK;