/*
 * create_president1.sql
 * Chapter 11, Oracle Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script builds a procedure to render a dynamic web page.
 */

-- Unremark for debugging script.
-- SET ECHO ON
SET SERVEROUTPUT ON SIZE 1000000

-- Delete any conflicting objects.
BEGIN

  -- Loop through and delete table if it exists.
  FOR i IN (SELECT   table_name
            FROM     user_tables
            WHERE    table_name = 'PRESIDENT' ) LOOP
            
    -- Use NDS to drop the dependent object type.
    EXECUTE IMMEDIATE 'DROP TABLE president';
    
  END LOOP;
  
  -- Loop through and delete table if it exists.
  FOR i IN (SELECT   sequence_name
            FROM     user_sequences
            WHERE    sequence_name = 'PRESIDENT_S1' ) LOOP
            
    -- Use NDS to drop the dependent object type.
    EXECUTE IMMEDIATE 'DROP SEQUENCE president_s1';
    
  END LOOP;
  
END;
/

-- Create table for dynamic web page display.
CREATE TABLE president
( president_id      NUMBER
, last_name         VARCHAR2(20 CHAR)
, first_name        VARCHAR2(20 CHAR)
, middle_name       VARCHAR2(20 CHAR)
, term_start        NUMBER
, term_end          NUMBER
, party             VARCHAR2(24 CHAR)
, CONSTRAINT pk_p1  PRIMARY KEY (president_id));

-- Create sequence for primary key numbering.
CREATE SEQUENCE president_s1
  INCREMENT BY   1
  START WITH     1
  ORDER;

-- Seed data for Chapter 11.
INSERT INTO president
VALUES ( president_s1.nextval
       ,'Washington'
       ,'George'
       ,''
       , 1789 
       , 1797 
       ,'Federalist');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Adams'
       ,'John'
       ,''
       , 1797 
       , 1801 
       ,'Federalist');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Jefferson'
       ,'Thomas'
       ,''
       , 1801 
       , 1809 
       ,'Democratic-Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Madison'
       ,'James'
       ,''
       , 1809 
       , 1817 
       ,'Democratic-Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Monroe'
       ,'James'
       ,''
       , 1817 
       , 1825 
       ,'Democratic-Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Adams'
       ,'John'
       ,'Quincy'
       , 1825 
       , 1829 
       ,'Democratic-Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Jackson'
       ,'Andrew'
       ,''
       , 1829 
       , 1837 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Van Buren'
       ,'Martin'
       ,''
       , 1837 
       , 1841 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Harrison'
       ,'William'
       ,'Henry'
       , 1841 
       , 1841 
       ,'Whig');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Tyler'
       ,'John'
       ,''
       , 1841 
       , 1845 
       ,'Whig');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Polk'
       ,'James'
       ,''
       , 1845 
       , 1849 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Taylor'
       ,'Zachary'
       ,''
       , 1849 
       , 1850 
       ,'Whig');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Fillmore'
       ,'Millard'
       ,''
       , 1850 
       , 1853 
       ,'Whig');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Pierce'
       ,'Franklin'
       ,''
       , 1853 
       , 1857 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Buchanan'
       ,'James'
       ,''
       , 1857 
       , 1861 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Lincoln'
       ,'Abraham'
       ,''
       , 1861 
       , 1864 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Johnson'
       ,'Andrew'
       ,''
       , 1865 
       , 1869 
       ,'Union');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Grant'
       ,'Ulysses'
       ,'S.'
       , 1869 
       , 1877 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Hayes'
       ,'Rutherford'
       ,'B.'
       , 1877 
       , 1881 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Garfield'
       ,'James'
       ,''
       , 1881 
       , 1881 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Arthur'
       ,'Chester'
       ,''
       , 1881 
       , 1885 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Cleveland'
       ,'Grover'
       ,''
       , 1885 
       , 1889 
       ,'Democratic');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Harrison'
       ,'Benjamin'
       ,''
       , 1889 
       , 1893 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Cleveland'
       ,'Grover'
       ,''
       , 1893 
       , 1897 
       ,'Democratic');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'McKinley'
       ,'William'
       ,''
       , 1897 
       , 1901 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Roosevelt'
       ,'Theodore'
       ,''
       , 1901 
       , 1909 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Taft'
       ,'William'
       ,'H.'
       , 1909 
       , 1913 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Wilson'
       ,'Woodrow'
       ,''
       , 1913 
       , 1921 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Coolidge'
       ,'Calvin'
       ,''
       , 1923 
       , 1929 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Hoover'
       ,'Herbert'
       ,''
       , 1929 
       , 1933 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Roosevelt'
       ,'Franklin'
       ,'D.'
       , 1933 
       , 1945 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Truman'
       ,'Harry'
       ,''
       , 1945 
       , 1953 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Eisenhower'
       ,'Dwight'
       ,''
       , 1953 
       , 1961 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Kennedy'
       ,'John'
       ,'F.'
       , 1961 
       , 1963 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Johnson'
       ,'Lyndon'
       ,''
       , 1963 
       , 1969 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Nixon'
       ,'Richard'
       ,'M.'
       , 1969 
       , 1974 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Ford'
       ,'Gerald'
       ,'R.'
       , 1974 
       , 1977 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Carter'
       ,'Jimmy'
       ,''
       , 1977 
       , 1981 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Reagan'
       ,'Ronald'
       ,''
       , 1981 
       , 1989 
       ,'');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Bush'
       ,'George'
       ,'H.W.'
       , 1989 
       , 1993 
       ,'Republican');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Clinton'
       ,'William'
       ,''
       , 1993
       , 2001 
       ,'Democrat');

INSERT INTO president
VALUES ( president_s1.nextval
       ,'Bush'
       ,'George'
       ,'W.'
       , 2001 
       , 2009 
       ,'Republican');

-- Commit the entries to the PRESIDENT table.
COMMIT;

-- Create a PL/SQL Toolkit generated webpage.
CREATE OR REPLACE PROCEDURE President1
( years   OWA_UTIL.IDENT_ARR ) AS

  CURSOR get_presidents 
  ( term_start_in NUMBER
  , term_end_in   NUMBER ) IS

    SELECT   president_id pres_number
    ,        first_name||' '||middle_name||' '||last_name pres_name
    ,        term_start||'-'||term_end tenure
    FROM     president
    WHERE    term_start BETWEEN term_start_in AND term_end_in
    OR       term_end BETWEEN term_start_in AND term_end_in;

BEGIN

  -- Set HTML page rendering tags.
  htp.htmlopen;
  htp.headopen;
  htp.htitle('President1');
  htp.headclose;
  htp.bodyopen;
  htp.line;
  
  -- Use PL/SQL Toolkit to format the page.
  htp.tableopen(cborder     => 2
               ,cattributes => 'style=background-color:feedb8');
    htp.tablerowopen;
      htp.tabledata(cvalue      => '#'
                   ,calign      => 'center'
                   ,cattributes => 'style=color:#336699
                                          background-color:#cccc99
                                          font-weight:bold
                                          width=50');
      htp.tabledata(cvalue      => 'NAME'
                   ,calign      => 'center'
                   ,cattributes => 'style=color:#336699
                                          background-color:#cccc99
                                          font-weight:bold
                                          width=200');
      htp.tabledata(cvalue      => 'TENURE'
                   ,calign      => 'center'
                   ,cattributes => 'style=color:#336699
                                          background-color:#cccc99
                                          font-weight:bold
                                          width=100');
    htp.tablerowclose;

  -- Use a loop to collect the data.
  FOR i IN get_presidents(years(1),years(2)) LOOP

    htp.tablerowopen;
      htp.tabledata(cvalue      => i.pres_number
                   ,calign      => 'center'
                   ,cattributes => 'style=background-color:#f7f7e7');
      htp.tabledata(cvalue      => i.pres_name
                   ,calign      => 'left'
                   ,cattributes => 'style=background-color:#f7f7e7');
      htp.tabledata(cvalue      => i.tenure
                   ,calign      => 'center'
                   ,cattributes => 'style=background-color:#f7f7e7');
    htp.tablerowclose;

  END LOOP;

  -- Close the table. 
  htp.tableclose;

  -- Print a line and close body and page.
  htp.line;
  htp.bodyclose;
  htp.htmlclose;
 
END president1;
/
