/*
 * create_user.sql
 * Chapter 12, Expert PL/SQL Programming
 * by Ron Hardman and Mike McLaughlin
 *
 * This script creates the plsql user for chapter 12.
 *   You must run this script as SYS or
 *   SYSTEM as SYSDBA.  The script can be rerun.
 *
 * The plsql user is created using the USERS and TEMP tablespace.
 *   If these tablespaces do not exist in your environment, change
 *   the script to use an appropriate tablespaces already created.
 *
 * Modify the conn_string value below, providing your Net Service
 *   Name if not using your default.
 *
 * The value for file_loc must be the location where your Data Pump
 *   export dump files are located.
 *
 */

DEF conn_string = plsql2/oracle@book
DEF username = plsql2
DEF default_ts = USERS
DEF temp_ts = TEMP

SET FEEDBACK OFF SERVEROUTPUT ON VERIFY OFF TERMOUT OFF

SPOOL create_user2.log

DECLARE
   v_count       INTEGER        := 0;
   v_statement   VARCHAR2 (500);
BEGIN

   -- Drop the user if it exists

   SELECT COUNT (1)
     INTO v_count
     FROM dba_users
    WHERE username = UPPER ('&username');

   IF v_count != 0
   THEN
      EXECUTE IMMEDIATE ('DROP USER &username CASCADE');
   END IF;

   v_count := 0;


   -- Create the user

   v_statement :=
         'CREATE USER &username IDENTIFIED BY oracle'
      || ' DEFAULT TABLESPACE &default_ts'
      || ' TEMPORARY TABLESPACE &temp_ts'
      || ' QUOTA UNLIMITED ON &default_ts'
      || ' ACCOUNT UNLOCK';

   EXECUTE IMMEDIATE (v_statement);

   -- Grant permissions
   EXECUTE IMMEDIATE ('GRANT connect, resource TO &username');
   EXECUTE IMMEDIATE ('GRANT create table TO &username');
   EXECUTE IMMEDIATE ('GRANT create procedure TO &username');
   EXECUTE IMMEDIATE ('GRANT create ANY directory, drop ANY directory TO &username');
   EXECUTE IMMEDIATE ('GRANT EXP_FULL_DATABASE TO &username');
   EXECUTE IMMEDIATE ('GRANT READ, WRITE ON DIRECTORY EXPORT_DIR TO &username');

   DBMS_OUTPUT.put_line ('	');
   DBMS_OUTPUT.put_line ('User &username created successfully');
   DBMS_OUTPUT.put_line ('	');


EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line (SQLERRM);
      DBMS_OUTPUT.put_line ('	');
END;
/

SET FEEDBACK ON TERMOUT ON

PROMPT
PROMPT Connecting to user &username
PROMPT

CONN &conn_string

SPOOL OFF
