 /*
 * compression.sql
 * Chapter 12, Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script demonstrates how to use the UTL_COMPRESS package
 *  to compress files on the file server.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL compression.log

SET ECHO OFF
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF
SET SERVEROUTPUT ON


CREATE TABLE compress_images (
                              ID NUMBER(10),
                              IMAGE BFILE);

-- FTP the book_cover.jpg image from the chapter12/files directory
--  to the directory specified in the user creation script.

INSERT INTO compress_images
VALUES (1, BFILENAME('IMAGE_DIR', 'book_cover.jpg'));

COMMIT;


DECLARE
   v_bfile_original BFILE;
   v_blob_compress BLOB := TO_BLOB('a');
   v_blob_uncompress BLOB := TO_BLOB('a');
   v_length PLS_INTEGER;
BEGIN
  -- Get the length of the image prior to compression
  SELECT DBMS_LOB.GETLENGTH(image)
  INTO v_length
  FROM compress_images;
  
  -- Display the length
  DBMS_OUTPUT.PUT(CHR(10));
  DBMS_OUTPUT.PUT_LINE('Before compression length: '|| v_length);
  
  -- Get the pointer
  SELECT image
  INTO v_bfile_original
  FROM compress_images;
  
  -- Open the bfile for read
  DBMS_LOB.OPEN(v_bfile_original);
  
  -- Compress the image
  UTL_COMPRESS.LZ_COMPRESS (src => v_bfile_original, 
                            dst => v_blob_compress);
  -- Display the length
  v_length := DBMS_LOB.GETLENGTH(v_blob_compress);
  DBMS_OUTPUT.PUT(CHR(10));
  DBMS_OUTPUT.PUT_LINE('Compressed length: '|| v_length);
  
  -- Uncompress the blob
  UTL_COMPRESS.LZ_UNCOMPRESS (v_blob_compress, 
                              v_blob_uncompress);
  
  -- Display the length
  v_length := DBMS_LOB.GETLENGTH(v_blob_uncompress);
  DBMS_OUTPUT.PUT(CHR(10));
  DBMS_OUTPUT.PUT_LINE('Uncompressed length: '|| v_length);
  
  -- Close the bfile and free temporary lobs
  DBMS_LOB.CLOSE(v_bfile_original);
  DBMS_LOB.FREETEMPORARY(v_blob_compress);
  DBMS_LOB.FREETEMPORARY(v_blob_uncompress);

END;
/ 

SPOOL OFF