 /*
 * crypto_aes.sql
 * Chapter 12, Expert PL/SQL Programming
 * by Ron Hardman and Michael McLaughlin
 *
 * This script demonstrates how to use the DBMS_CRYPTO package to 
 *  encrypt and decrypt data.
 */

CLEAR BREAKS
CLEAR COLUMNS
CLEAR COMPUTES

TTITLE OFF

SPOOL crypto_aes.log

SET ECHO OFF
SET FEEDBACK ON
SET NULL '<Null>'
SET PAGESIZE 999
SET PAUSE OFF
SET TERM ON
SET TIME OFF
SET TIMING OFF
SET VERIFY OFF
SET SERVEROUTPUT ON


CREATE OR REPLACE PROCEDURE crypto_aes (
   i_plaintext IN VARCHAR2)
IS
   v_ciphertext RAW(500);
   v_deciphered VARCHAR2(500 CHAR);
   v_plaintext RAW(200) := UTL_I18N.STRING_TO_RAW(
                                      i_plaintext,
                                      'AL32UTF8');
   v_type BINARY_INTEGER := DBMS_CRYPTO.AES_CBC_PKCS5;
   v_key RAW(128) := UTL_I18N.STRING_TO_RAW(
            DBMS_CRYPTO.RANDOMBYTES(16),  --generates 128 bit key..128/8
            'AL32UTF8');
BEGIN
   DBMS_OUTPUT.PUT(CHR(10));
   DBMS_OUTPUT.PUT_LINE('Encrypted using ENCRYPT_AES + CHAIN_CBC + PAD_PKCS5');
   DBMS_OUTPUT.PUT_LINE('===================================================');
   DBMS_OUTPUT.PUT(CHR(10));
   DBMS_OUTPUT.PUT_LINE('PLAINTEXT: '||i_plaintext);
   v_ciphertext := DBMS_CRYPTO.ENCRYPT(v_plaintext,
                                       v_type,
                                       v_key);
   DBMS_OUTPUT.PUT(CHR(10));
   DBMS_OUTPUT.PUT_LINE('CIPHERTEXT: '||v_ciphertext);
   DBMS_OUTPUT.PUT(CHR(10));
   DBMS_OUTPUT.PUT(CHR(10));
   
   v_deciphered := UTL_I18N.RAW_TO_CHAR(
                       DBMS_CRYPTO.DECRYPT(v_ciphertext,
                                           v_type,
                                           v_key));
   
   DBMS_OUTPUT.PUT(CHR(10));
   DBMS_OUTPUT.PUT_LINE('Decrypt the cipertext');
   DBMS_OUTPUT.PUT_LINE('=====================');
   DBMS_OUTPUT.PUT(CHR(10));
   DBMS_OUTPUT.PUT_LINE('DECIPHERED: '||v_deciphered);
END crypto_aes;
/
   
   
SET SERVEROUTPUT ON
exec crypto_aes('Expert PLSQL')

SPOOL OFF